<?PHP
/**
 * This file offers a class for handling outputs and logging.
 *
 * @author Joshua Ramon Enslin <joshua@museum-digital.de>
 */
declare(strict_types = 1);

/**
 * Class for handling outputs and logs for museum-digital's background tools.
 */
final class MDOutputHandler {

    private const FLUSH_TO_LOGFILE_AFTER = 100;

    public static bool $disable_completely = false;

    /** @var integer */
    private int $_verbosity = 0;
    /** @var string */
    private string $_context = "";
    /** @var string */
    private string $_logfile;
    /** @var integer */
    private $_counter = 0;
    /** @var boolean */
    private bool $_file_logging = false;
    /** @var float */
    private float $_startTime;
    /** @var boolean */
    public bool $show_operation_number = true;

    /** @var string[] */
    private array $_log_queue = [];
    /** @var boolean */
    private bool $_logfile_status_checked = false;

    /**
     * Simple echo that is silent during $disable_completely.
     *
     * @param string $input Input text.
     *
     * @return void
     */
    public static function echo(string $input):void {

        if (self::$disable_completely === true) return;
        echo $input;

    }

    /**
     * Writes log file contents to log file.
     *
     * @return void
     */
    public function flush_to_logfile():void {

        echo "Will flush to log file" . PHP_EOL . PHP_EOL . PHP_EOL;

        if ($this->_logfile_status_checked === false) {

            if (!\is_dir(__DIR__ . "/../logs")) {
                MD_STD::mkdir(__DIR__ . "/../logs", 0775);
            }

            \touch($this->_logfile);
            $this->_logfile_status_checked = true;

        }

        \file_put_contents($this->_logfile, implode(PHP_EOL, $this->_log_queue), FILE_APPEND);

        $this->_log_queue = [];

    }

    /**
     * Function human_filesize translates byte-level filesizes to human readable ones.
     * Thanks to Jeffrey Sambells http://jeffreysambells.com/2012/10/25/human-readable-filesize-php
     *
     * @param integer $bytes    A file size, e.g. returned from filesize().
     * @param integer $decimals Number of decimal digits to allow.
     *
     * @return string
     */
    private function _human_filesize(int $bytes, int $decimals = 2):string {

        $size = ['B', 'kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
        $factor = floor((strlen((string)$bytes) - 1) / 3);
        return sprintf("%.{$decimals}f", $bytes / pow(1024, $factor)) . $size[$factor];

    }

    /**
     * Setter function for the verbosity option.
     *
     * @param integer $input New value of boolean value verbosity.
     *
     * @return void
     */
    public function setVerbosity(int $input = 0):void {

        $this->_verbosity = $input;

    }

    /**
     * Setter function for the additional context description.
     *
     * @param string $input New value.
     *
     * @return void
     */
    public function setContext(string $input = ""):void {

        $this->_context = $input;

    }

    /**
     * Setter function for file logging.
     *
     * @param boolean $input New value.
     *
     * @return void
     */
    public function setFileLogging(bool $input = false):void {

        $this->_file_logging = $input;

    }

    /**
     * Main function of this class.
     * Formats a message uniformly and writes it to the log, maybe also to stdout.
     *
     * @param string          $msg        Message to log.
     * @param MDConsoleStatus $statusCode Status code.
     *
     * @return void
     */
    public function toLog(string $msg, MDConsoleStatus $statusCode = MDConsoleStatus::NOTICE):void {

        if (self::$disable_completely === true) return;

        // Get parts for generating the message to log.
        ++$this->_counter;              // Increment counter
        $date = \date("Y-m-d H:i:s");    // Get current date & time.

        $statusSymbol = match($statusCode) {
            MDConsoleStatus::SKIPPED => "-",
            MDConsoleStatus::UPDATE => "U",
            MDConsoleStatus::ERROR => "E",
            MDConsoleStatus::DELETION => "D",
            MDConsoleStatus::NOTICE => "N",
        };

        if ($this->_context !== "") $contextStr = "{$this->_context} │ ";
        else $contextStr = "";

        // Generate the message to log.
        $message = "$statusSymbol │ {$contextStr}#" . sprintf("%1$07s", $this->_counter) . " │ " . \sprintf("%1$7s", $this->_human_filesize(\memory_get_usage())) . " │ {$date} │ {$msg}" . PHP_EOL;

        if ($this->_file_logging === true) {
            $this->_log_queue[] = $message;
            if (count($this->_log_queue) === self::FLUSH_TO_LOGFILE_AFTER) {
                $this->flush_to_logfile();
            }
        }

        if (($this->_verbosity > 0 && \in_array($statusCode, [MDConsoleStatus::UPDATE, MDConsoleStatus::DELETION, MDConsoleStatus::ERROR], true)) || $this->_verbosity === 2) echo $message;

        if ($this->show_operation_number === true && $this->_counter % 250 === 0) {
            echo '―――> ' . \sprintf("%1$07d", $this->_counter) . " operations processed" . PHP_EOL;
            echo str_replace("_", "―", sprintf("%'_80s", "_")) . PHP_EOL;
        }

    }

    /**
     * Constructor function that sets defaults and ensures the logfile exists.
     */
    public function __construct() {

        $this->_startTime = \microtime(true);
        $this->setVerbosity();
        $this->setContext();
        $this->setFileLogging();
        $this->_logfile   = __DIR__ . "/../logs/" . \date("Ymd_His") . ".txt";

        // Ensure the log directory exists

        $this->toLog("Activated logger.");

    }

    /**
     * Destructor.
     *
     * @return void
     */
    public function __destruct() {

        if (self::$disable_completely === true) return;

        if ($this->_verbosity < 0) return;

        if (!empty($this->_log_queue)) {
            $this->flush_to_logfile();
        }

        $endtime = \microtime(true);
        $this->toLog("Finished operation. Processing duration: " . \strval($endtime - $this->_startTime));

    }
}


